==> libstpk-0.9.20.0.tar.gz: libstpk-0.9.20.0/demo/unit/fft_data2.f90 <==
program fft_test
! fft 롼Ѥ, ƥȥǡ饹ڥȥʬۤ׻롼

  use file_operate
  use Basis
  use ffts
  use Math_Const
  use max_min

  implicit none

  integer :: na, nl
  integer :: i, j, l, emax, loop
  integer, dimension(5) :: prim

  real :: period
  real, allocatable :: sp(:,:), ren(:,:)
  real, allocatable :: val(:,:)

  complex, allocatable :: co(:,:), ccval(:,:), crval(:,:)
  complex, allocatable, dimension(:,:) :: omegar, omegai, omeganr, omegani

  character(20), allocatable, dimension(:,:) :: cval
  character(20) :: fom, fomc
  character(100) :: fname, foname
  character(1000) :: ctitle

  write(*,*) "Input a reading file name."
  write(*,*) "If there is no file, input 'dummy'"
  read(*,*) fname

  if(trim(fname)/='dummy')then
     write(*,*) "Input the reading number of array."
     read(*,*) na

!-- reading file

     nl=line_number_counter( trim(fname) )

!-- checking "nl" (If nl is not prim factor of 2, 
!-- nl is forced to become prim factor of 2.)

write(*,*) "nl check", nl
     if(mod(nl,2)/=0)then
        nl=nl-1
        write(*,*) "message : colum number is not even number."
        write(*,*) "nl is FORCED to become even number."
     end if

write(*,*) "nl check", nl
     allocate(cval(na,nl))
     allocate(val(nl,na))
     allocate(ccval(nl,na))
     allocate(crval(nl,na))
     allocate(ren(nl,na))

     call read_file_text( trim(fname), na, nl, cval )

!-- converting from character to float

     do j=1,nl
        do i=1,na
           val(j,i)=c2r_convert( trim(adjustl(cval(i,j))) )
        end do
     end do

  else

     write(*,*) "Automatically, sample file is producted."

     na=3
     nl=100

     allocate(val(nl,na))
     allocate(ccval(nl,na))
     allocate(crval(nl,na))
     allocate(ren(nl,na))

     do i=1,nl
        val(i,1)=1.0+sin(2.0*pi*(real(i-1)/real(nl)))
        val(i,2)=1.0+sin(4.0*pi*(real(i-1)/real(nl)))
        val(i,3)=1.0+sin(6.0*pi*(real(i-1)/real(nl)))
     end do

  end if

!-- NOTICE : array order is opposite (char : na, nl) (float : nl, na)
  allocate(co(nl,na))
  allocate(sp(nl,na))

!-- calculating prim factors for nl

  call prim_calc( nl, prim(1:4), prim(5) )
  write(*,*) "result prim", prim(1), prim(2), prim(3), prim(4), prim(5)

!-- calculating rotating arrays

  allocate(omegar(0:nl-1,0:nl-1))
  allocate(omegai(0:nl-1,0:nl-1))
  allocate(omeganr(0:nl-1,0:nl-1))
  allocate(omegani(0:nl-1,0:nl-1))

  call rotate_array_f()
  call rotate_calc( nl, 'r', (/prim(1),prim(2),prim(3),prim(4),prim(5)/),  &
  &                 omegar(0:prim(5)-1,0:prim(5)-1),  &
  &                 omeganr(0:nl-1,0:nl-1) )
  call rotate_calc( nl, 'i', (/prim(1),prim(2),prim(3),prim(4),prim(5)/),  &
  &                 omegai(0:prim(5)-1,0:prim(5)-1),  &
  &                 omegani(0:nl-1,0:nl-1) )


  do j=1,na
     do i=1,nl
        ccval(i,j)=val(i,j)
     end do
     call ffttp_1d( nl, ccval(1:nl,j), co(1:nl,j),  &
  &                 csign='r', prim='o', prim_fact=prim,  &
  &                 omega_fix=omegar, omegan_fix=omeganr )
     call ffttp_1d( nl, co(1:nl,j), crval(1:nl,j), prim='o',  &
  &                 csign='i', prim_fact=prim,  &
  &                 omega_fix=omegai, omegan_fix=omegani )
     do i=2,nl/2-1
        sp(i,j)=abs(co(i,j))+abs(co(nl-(i-2),j))
     end do
     sp(1,j)=abs(co(1,j))
     sp(nl/2,j)=abs(co(nl/2,j))
!     sp(1,j)=abs(real(co(1,j)))
!     sp(nl/2+1,j)=abs(aimag(co(1,j)))
     do i=1,nl
        ren(i,j)=real(crval(i,j))
     end do
  end do

  do i=1,nl
     write(*,*) val(i,1), ren(i,1), val(i,2), ren(i,2), val(i,3), ren(i,3)
  end do

  write(*,*) "### Finished calculating."
  write(*,*) "Input the output file name."
  read(*,*) foname

  write(*,*) "Input the calculating period."
  write(*,*) "(data period)"
  read(*,*) period

!-- formatting of output

  fom='(1P,'//trim(adjustl(i2c_convert( na+3 )))//'E14.5)'
write(*,*) "check a format", trim(fom)

  ctitle='wave-number frequency period '

  do i=1,na
     ctitle=trim(ctitle)//' val'
  end do

  fomc='(a'//trim(adjustl(i2c_convert( len_trim(ctitle) )))//')'

!-- output another file

  open(unit=11,file=trim(foname),status='unknown')

     write(11,trim(fomc)) trim(ctitle)
     write(11,trim(fomc)) trim(ctitle)

     write(11,trim(fom)) 0.0, 0.0, period, (sp(1,i),i=1,na)

     do j=2,nl/2
        write(11,trim(fom)) real(j-1), real(j-1)/period, period/real(j-1),  &
  &                         (sp(j,i),i=1,na)
     end do

  close(unit=11,status='keep')

end program

==> libstpk-0.9.30.0.tar.gz: libstpk-0.9.30.0/demo/unit/fft_data2.f90 <==
program fft_test
! fft ルーチンを用いて, テキストデータからスペクトル分布を計算するルーチン

  use file_operate
  use Basis
  use ffts
  use Math_Const
  use max_min

  implicit none

  integer :: na, nl
  integer :: i, j, l, emax, loop
  integer, dimension(5) :: prim

  real :: period
  real, allocatable :: sp(:,:), ren(:,:)
  real, allocatable :: val(:,:)

  complex, allocatable :: co(:,:), ccval(:,:), crval(:,:)
  complex, allocatable, dimension(:,:) :: omegar, omegai, omeganr, omegani

  character(20), allocatable, dimension(:,:) :: cval
  character(20) :: fom, fomc
  character(100) :: fname, foname
  character(1000) :: ctitle

  write(*,*) "Input a reading file name."
  write(*,*) "If there is no file, input 'dummy'"
  read(*,*) fname

  if(trim(fname)/='dummy')then
     write(*,*) "Input the reading number of array."
     read(*,*) na

!-- reading file

     nl=line_number_counter( trim(fname) )

!-- checking "nl" (If nl is not prim factor of 2, 
!-- nl is forced to become prim factor of 2.)

write(*,*) "nl check", nl
     if(mod(nl,2)/=0)then
        nl=nl-1
        write(*,*) "message : colum number is not even number."
        write(*,*) "nl is FORCED to become even number."
     end if

write(*,*) "nl check", nl
     allocate(cval(na,nl))
     allocate(val(nl,na))
     allocate(ccval(nl,na))
     allocate(crval(nl,na))
     allocate(ren(nl,na))

     call read_file_text( trim(fname), na, nl, cval )

!-- converting from character to float

     do j=1,nl
        do i=1,na
           val(j,i)=c2r_convert( trim(adjustl(cval(i,j))) )
        end do
     end do

  else

     write(*,*) "Automatically, sample file is producted."

     na=3
     nl=100

     allocate(val(nl,na))
     allocate(ccval(nl,na))
     allocate(crval(nl,na))
     allocate(ren(nl,na))

     do i=1,nl
        val(i,1)=1.0+sin(2.0*pi*(real(i-1)/real(nl))+(1.0/6.0)*pi)
        val(i,2)=1.0+sin(4.0*pi*(real(i-1)/real(nl))+(1.0/6.0)*pi)
        val(i,3)=1.0+sin(6.0*pi*(real(i-1)/real(nl))+(1.0/6.0)*pi)
        !-- Note: sin(theta + alpha) = cos(theta + alpha - pi/2)
     end do

  end if

!-- NOTICE : array order is opposite (char : na, nl) (float : nl, na)
  allocate(co(nl,na))
  allocate(sp(nl,na))

!-- calculating prim factors for nl

  call prim_calc( nl, prim(1:4), prim(5) )
  write(*,*) "result prim", prim(1), prim(2), prim(3), prim(4), prim(5)

!-- calculating rotating arrays

  allocate(omegar(0:nl-1,0:nl-1))
  allocate(omegai(0:nl-1,0:nl-1))
  allocate(omeganr(0:nl-1,0:nl-1))
  allocate(omegani(0:nl-1,0:nl-1))

  call rotate_array_f()
  call rotate_calc( nl, 'r', (/prim(1),prim(2),prim(3),prim(4),prim(5)/),  &
  &                 omegar(0:prim(5)-1,0:prim(5)-1),  &
  &                 omeganr(0:nl-1,0:nl-1) )
  call rotate_calc( nl, 'i', (/prim(1),prim(2),prim(3),prim(4),prim(5)/),  &
  &                 omegai(0:prim(5)-1,0:prim(5)-1),  &
  &                 omegani(0:nl-1,0:nl-1) )


  do j=1,na
     do i=1,nl
        ccval(i,j)=val(i,j)
     end do
     call ffttp_1d( nl, ccval(1:nl,j), co(1:nl,j),  &
  &                 csign='r', prim='o', prim_fact=prim,  &
  &                 omega_fix=omegar, omegan_fix=omeganr )
!-- 波数取り出しチェック (ここから)
co(1:j,j)=0.0
co(j+2:nl-j,j)=0.0
if(j>1)then
   co(nl-(j-2):nl,j)=0.0
end if
!-- 波数取り出しチェック (ここまで)
     call ffttp_1d( nl, co(1:nl,j), crval(1:nl,j), prim='o',  &
  &                 csign='i', prim_fact=prim,  &
  &                 omega_fix=omegai, omegan_fix=omegani )
     do i=2,nl/2-1
        sp(i,j)=abs(co(i,j))+abs(co(nl-(i-2),j))
     end do
     !-- Note: phase "alpha" in cos(n theta + alpha) is calculated by atan(Im(co)/Re(co))
     !-- cos(n theta + alpha) = sin(n theta + alpha + pi/2). 
write(*,*) "co check", co(j+1,j), co(nl-(j-1),j), j, atan2(aimag(co(j+1,j)),real(co(j+1,j)))*180.0/pi
write(*,*) "Parceval check", real(co(1,j))**2, 2.0*(real(co(j+1,j))**2+aimag(co(j+1,j))**2)
     sp(1,j)=abs(co(1,j))
     sp(nl/2,j)=abs(co(nl/2,j))
!     sp(1,j)=abs(real(co(1,j)))
!     sp(nl/2+1,j)=abs(aimag(co(1,j)))
     do i=1,nl
        ren(i,j)=real(crval(i,j))
     end do
  end do

  stop  ! check
  do i=1,nl
     write(*,'(1P7E16.8)') real(i), val(i,1), ren(i,1), val(i,2), ren(i,2), val(i,3), ren(i,3)
  end do

  write(*,*) "### Finished calculating."
  write(*,*) "Input the output file name."
  read(*,*) foname

  write(*,*) "Input the calculating period."
  write(*,*) "(data period)"
  read(*,*) period

!-- formatting of output

  fom='(1P,'//trim(adjustl(i2c_convert( na+3 )))//'E14.5)'
write(*,*) "check a format", trim(fom)

  ctitle='wave-number frequency period '

  do i=1,na
     ctitle=trim(ctitle)//' val'
  end do

  fomc='(a'//trim(adjustl(i2c_convert( len_trim(ctitle) )))//')'

!-- output another file

  open(unit=11,file=trim(foname),status='unknown')

     write(11,trim(fomc)) trim(ctitle)
     write(11,trim(fomc)) trim(ctitle)

     write(11,trim(fom)) 0.0, 0.0, period, (sp(1,i),i=1,na)

     do j=2,nl/2
        write(11,trim(fom)) real(j-1), real(j-1)/period, period/real(j-1),  &
  &                         (sp(j,i),i=1,na)
     end do

  close(unit=11,status='keep')

end program
